searchData={"content_type":"text/markdown","items":[{"doc":"Functions for integrating Kafka tracing with DataStreams.","ref":"Datadog.DataStreams.Integrations.Kafka.html","title":"Datadog.DataStreams.Integrations.Kafka","type":"module"},{"doc":"Because Elixir does not include a `context` grab bag to pass\naround, we use the OpenTelemetry context to store the current\nDataStreams pathway. If you are not using OpenTelemetry, or\nhave a special fan in or fan out situation, you can use the\nrespective functions that take a pathway as an argument.\n\nIf you have a basic one message in one message out situation,\n_and_ you have OpenTelemetry already covering your application,\nyou can use the `trace_produce/1` and `trace_consume/2` functions.\n\n    require OpenTelemetry.Tracer, as: Tracer\n\n    alias Datadog.DataStreams.Integrations.Kafka, as: DataStreamsKafka\n\n    @doc \"Handles a message from Kafka. Receives a message map with partition, topic, and headers.\"\n    @spec handle_message(map()) :: :ok\n    def handle_message(message) do\n      # NOTE: This does not add the recommended Kafka span attributes.\n      Tracer.with_span \"#{message.topic} process\" do\n        DataStreamsKafka.trace_consume(message, \"my_consumer_group\")\n\n        # Do work\n\n        Tracer.with_span \"#{new_message.topic} produce\" do\n          new_message\n          |> DataStreamsKafka.trace_produce()\n          |> send_to_kafka()\n        end\n      end\n    end","ref":"Datadog.DataStreams.Integrations.Kafka.html#module-usage","title":"Usage - Datadog.DataStreams.Integrations.Kafka","type":"module"},{"doc":"Traces a Kafka message being consumed. Requires the current Kafka\nconsumer group. Uses the pathway in the current\n`Datadog.DataStreams.Context`.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_consume/2","title":"Datadog.DataStreams.Integrations.Kafka.trace_consume/2","type":"function"},{"doc":"Traces a Kafka message being consumed. Requires the current Kafka\nconsumer group.\n\nDo not pass the resulting pathway from this function to another call\nof `trace_consume_with_pathway`, as it will modify the pathway incorrectly.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_consume_with_pathway/3","title":"Datadog.DataStreams.Integrations.Kafka.trace_consume_with_pathway/3","type":"function"},{"doc":"Traces a Kafka message being produced. Uses the pathway in the\ncurrent `Datadog.DataStreams.Context`. Returns a new message with\nthe pathway encoded in the header values.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_produce/1","title":"Datadog.DataStreams.Integrations.Kafka.trace_produce/1","type":"function"},{"doc":"Traces a Kafka message being produced. Returns the new message with the\npathway encoded in the header values, as well as the new pathway.","ref":"Datadog.DataStreams.Integrations.Kafka.html#trace_produce_with_pathway/2","title":"Datadog.DataStreams.Integrations.Kafka.trace_produce_with_pathway/2","type":"function"},{"doc":"Tracks Kafka produce events via their offset. This is used by Datadog\nto calculate the lag without requiring the consumer to be on and\nreading trace headers.","ref":"Datadog.DataStreams.Integrations.Kafka.html#track_consume/4","title":"Datadog.DataStreams.Integrations.Kafka.track_consume/4","type":"function"},{"doc":"Tracks Kafka produce events via their offset. This is used by Datadog\nto calculate the lag without requiring the consumer to be on and\nreading trace headers.","ref":"Datadog.DataStreams.Integrations.Kafka.html#track_produce/3","title":"Datadog.DataStreams.Integrations.Kafka.track_produce/3","type":"function"},{"doc":"A general map that contains the topic, partition, and headers atoms. This\nmatches the format of `Elsa.elsa_message` by default\n(and will work out of the box), though will need the topic and partition\nadded if you are using standard `:brod` (or `kpro`).","ref":"Datadog.DataStreams.Integrations.Kafka.html#t:message/0","title":"Datadog.DataStreams.Integrations.Kafka.message/0","type":"type"},{"doc":"This is a port of the [data-streams-go][dsg] library to Elixir.","ref":"Datadog.DataStreams.html","title":"Datadog.DataStreams","type":"module"},{"doc":"Configuration can be done via the `Config` module and your `config/` files.\nWe use OpenTelemetry configured values first, so if you are already using\nOpenTelemetry to instrument your application, your service name and\nenvironment should already be set.\n\n    import Config\n\n    config :opentelemetry, :resource,\n      name: \"my-elixir-service\",\n      env: \"production\"\n\nIf you are not using OpenTelemetry, you can set the service, environment,\nand primary tag via the `:data_streams` application like so:\n\n    import Config\n\n    config :data_streams, :metadata,\n      service: \"my-elixir-service\",\n      env: \"production\",\n      primary_tag: \"datacenter:d1\"\n\nOnce that is configured, you will also want to set the configuration for\naccessing the Datadog agent. This can be done via:\n\n    import Config\n\n    config :data_streams, :agent,\n      enabled?: true,\n      host: \"my-datadog-agent.local\",\n      port: 8125\n\n**By default, the agent is disabled and will not send data to Datadog**.\nWithout the `host` or `port` configured, we default to \"localhost:8125\".\n\nFor more information, view the `Datadog.DataStreams.Config` module.","ref":"Datadog.DataStreams.html#module-configuration","title":"Configuration - Datadog.DataStreams","type":"module"},{"doc":"Once installed, if `:dd_agent_streams` `:agent` is `enabled?`, the\n`Datadog.DataStreams.Aggregator` will start automatically and start sending\nmetrics. Just instrument your data pipelines!","ref":"Datadog.DataStreams.html#module-running","title":"Running - Datadog.DataStreams","type":"module"},{"doc":"- `Datadog.DataStreams.Integrations.Kafka`","ref":"Datadog.DataStreams.html#module-integrations","title":"Integrations - Datadog.DataStreams","type":"module"},{"doc":"Similar to the golang implementation, we export a couple of `:telemetry`\nmetrics that can (and should) be sent to Datadog. These can be done via\nhow ever you are sending your application telemetry metrics to Datadog.\n\nAvailable metrics include:\n\n- `datadog.datastreams.aggregator.payloads_in.count` - The number of\n  `Datadog.DataStreams.Aggregator.Point`s that were sent to the\n  `Datadog.DataStreams.Aggregator`.\n\n- `datadog.datastreams.aggregator.flushed_payloads.count` - The number of\n  successful payloads sent to Datadog.\n\n- `datadog.datastreams.aggregator.flushed_buckets.count` - The number of\n  successful 10 second buckets sent to Datadog.\n\n- `datadog.datastreams.aggregator.flush_errors.count` - The number of failed\n  requests to the Datadog agent (and dropped payloads).\n\n[dsg]: https://github.com/DataDog/data-streams-go","ref":"Datadog.DataStreams.html#module-telemetry","title":"Telemetry - Datadog.DataStreams","type":"module"},{"doc":"A `GenServer` instance responsible for aggregating many points of data\ntogether into 10 second buckets, and then sending them to the Datadog\nagent. It holds many structs in its memory, looking something like this:\n\n```mermaid\ngraph TD\n    aggregator[Datadog.DataStreams.Aggregator]\n    aggregator --> bucket[Datadog.DataStreams.Aggregator.Bucket]\n    bucket --> group[Datadog.DataStreams.Aggregator.Group]\n```\n\nWhen adding data, the calling code will create a new\n`Datadog.DataStreams.Aggregator.Point` which contains all of the needed\ndata. It will then call `Elixir.Datadog.DataStreams.Aggregator.add/1` to add that point of data to the\naggregator, where the aggregator will find (or create) a bucket that matches\nthe 10 second window for the point. It will then find (or create) a group in\nthat bucket based on the point's `hash`. Once the group is found, the\n`pathway_latency` and `edge_latency` `Datadog.Sketch` will be updated with\nthe new latency.\n\nEvery 10 seconds the aggregator will convert all non active (outside the 10\nsecond window) to a `Datadog.DataStreams.Payload`, encode it, and send it to\nthe Datadog agent. If there is an error sending the payload, the old payloads\nare still removed from memory, but the\n`datadog.datastreams.aggregator.flush_errors.count` telemetry metric is\nincremented.","ref":"Datadog.DataStreams.Aggregator.html","title":"Datadog.DataStreams.Aggregator","type":"module"},{"doc":"Adds new metrics to the aggregator.\n\nNote, this function will still return `:ok` if the aggregator is disabled.","ref":"Datadog.DataStreams.Aggregator.html#add/1","title":"Datadog.DataStreams.Aggregator.add/1","type":"function"},{"doc":"iex> :ok = Aggregator.add(%Aggregator.Point{})\n\n    iex> :ok = Aggregator.add(%Aggregator.Offset{})","ref":"Datadog.DataStreams.Aggregator.html#add/1-examples","title":"Examples - Datadog.DataStreams.Aggregator.add/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Datadog.DataStreams.Aggregator.html#child_spec/1","title":"Datadog.DataStreams.Aggregator.child_spec/1","type":"function"},{"doc":"Sends all stored data to the Datadog agent.","ref":"Datadog.DataStreams.Aggregator.html#flush/0","title":"Datadog.DataStreams.Aggregator.flush/0","type":"function"},{"doc":"iex> :ok = Aggregator.flush()","ref":"Datadog.DataStreams.Aggregator.html#flush/0-examples","title":"Examples - Datadog.DataStreams.Aggregator.flush/0","type":"function"},{"doc":"Starts a new `Elixir.Datadog.DataStreams.Aggregator` instance. This takes no options as it\nuses the global `Datadog.DataStreams.Config` module. It is also started\nby the `Datadog.DataStreams.Application` and should not need to be started\nmanually.","ref":"Datadog.DataStreams.Aggregator.html#start_link/1","title":"Datadog.DataStreams.Aggregator.start_link/1","type":"function"},{"doc":"Responsible for parsing application configuration to usable chunks\nin the application.","ref":"Datadog.DataStreams.Config.html","title":"Datadog.DataStreams.Config","type":"module"},{"doc":"Checks if the Datadog agent is enabled. By default it is disabled. Disabling\nwill cause the `Datadog.DataStreams.Aggregator` process to not start or send\ndata to the agent.","ref":"Datadog.DataStreams.Config.html#agent_enabled?/0","title":"Datadog.DataStreams.Config.agent_enabled?/0","type":"function"},{"doc":"iex> Config.agent_enabled?\n    false\n\n    iex> Application.put_env(:data_streams, :agent, enabled?: true)\n    ...> Config.agent_enabled?\n    true","ref":"Datadog.DataStreams.Config.html#agent_enabled?/0-examples","title":"Examples - Datadog.DataStreams.Config.agent_enabled?/0","type":"function"},{"doc":"Returns a full path to the Datadog agent, joining the given path string.\nIf this configuration is not set, we default to \"localhost:8126\".","ref":"Datadog.DataStreams.Config.html#agent_url/1","title":"Datadog.DataStreams.Config.agent_url/1","type":"function"},{"doc":"iex> Config.agent_url(\"/info\")\n    \"http://localhost:8126/info\"\n\n    iex> Application.put_env(:data_streams, :agent, [host: \"my-agent.local\", port: 1234])\n    ...> Config.agent_url(\"/info\")\n    \"http://my-agent.local:1234/info\"","ref":"Datadog.DataStreams.Config.html#agent_url/1-examples","title":"Examples - Datadog.DataStreams.Config.agent_url/1","type":"function"},{"doc":"Returns the configured environment tag for this application.\n\nFirst, it will try accessing the configured service for `:open_telemetry`.\nIf that is not set, it tries to use the service configured for\n`:data_streams`. If that is not set, it falls back to an empty string.\n\nNote, this will not pull open telemetry attributes set via environment\nvariables.","ref":"Datadog.DataStreams.Config.html#env/0","title":"Datadog.DataStreams.Config.env/0","type":"function"},{"doc":"iex> Application.put_env(:opentelemetry, :resource, service: %{env: \"production\"})\n    ...> Config.env()\n    \"production\"\n\n    iex> Application.put_env(:data_streams, :metadata, env: \"staging\")\n    ...> Config.env()\n    \"staging\"\n\n    iex> Config.env()\n    \"\"","ref":"Datadog.DataStreams.Config.html#env/0-examples","title":"Examples - Datadog.DataStreams.Config.env/0","type":"function"},{"doc":"Returns the configured primary tag for all `Datadog.DataStreams.Pathway`s.\n\nUsually this reflects a data center or some other top level partition for\nmetrics.\n\nIf this is not set, it will fall back to an empty string.","ref":"Datadog.DataStreams.Config.html#primary_tag/0","title":"Datadog.DataStreams.Config.primary_tag/0","type":"function"},{"doc":"iex> Application.put_env(:data_streams, :metadata, primary_tag: \"datacenter:d1\")\n    ...> Config.primary_tag()\n    \"datacenter:d1\"\n\n    iex> Config.primary_tag()\n    \"\"","ref":"Datadog.DataStreams.Config.html#primary_tag/0-examples","title":"Examples - Datadog.DataStreams.Config.primary_tag/0","type":"function"},{"doc":"Returns the configured service for this application.\n\nFirst, it will try accessing the configured service for `:open_telemetry`.\nIf that is not set, it tries to use the service configured for\n`:data_streams`. If that is not set, it falls back to\n\"unnamed-elixir-service\".\n\nNote, this will not pull open telemetry attributes set via environment\nvariables.","ref":"Datadog.DataStreams.Config.html#service/0","title":"Datadog.DataStreams.Config.service/0","type":"function"},{"doc":"iex> Application.put_env(:opentelemetry, :resource, service: %{name: \"my-elixir-service\"})\n    ...> Config.service()\n    \"my-elixir-service\"\n\n    iex> Application.put_env(:data_streams, :metadata, service: \"my-data-streams-service\")\n    ...> Config.service()\n    \"my-data-streams-service\"\n\n    iex> Config.service()\n    \"unnamed-elixir-service\"","ref":"Datadog.DataStreams.Config.html#service/0-examples","title":"Examples - Datadog.DataStreams.Config.service/0","type":"function"},{"doc":"Logic for linking current running container id to data stream traces.","ref":"Datadog.DataStreams.Container.html","title":"Datadog.DataStreams.Container","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Datadog.DataStreams.Container.html#child_spec/1","title":"Datadog.DataStreams.Container.child_spec/1","type":"function"},{"doc":"Returns the current container id.","ref":"Datadog.DataStreams.Container.html#get/0","title":"Datadog.DataStreams.Container.get/0","type":"function"},{"doc":"Attempts to return the container id from the provided file stream. Empty on failure.","ref":"Datadog.DataStreams.Container.html#parse_container_id/1","title":"Datadog.DataStreams.Container.parse_container_id/1","type":"function"},{"doc":"Attempts to return the container id from the cgroup path (`/proc/self/cgroup`). Empty on failure.","ref":"Datadog.DataStreams.Container.html#read_container_id/0","title":"Datadog.DataStreams.Container.read_container_id/0","type":"function"},{"doc":"Attempts to return the container id from the provided file path. Empty on failure.","ref":"Datadog.DataStreams.Container.html#read_container_id/1","title":"Datadog.DataStreams.Container.read_container_id/1","type":"function"},{"doc":"Starts the agent and stores the current container id in memory.","ref":"Datadog.DataStreams.Container.html#start_link/1","title":"Datadog.DataStreams.Container.start_link/1","type":"function"},{"doc":"This module deals with storing a `Datadog.DataStreams.Pathway`\nin process, so it can be propagated later on. In Golang, this is handled\nvia [`Context`](https://pkg.go.dev/context) which is a grab bag that is\npassed down the whole calling stack. We don't have an equivalent in Elixir,\nso we instead piggy back off of [`OpenTelemetry.Baggage`][OTB]. This\nessentially lets us do the same thing without needing to manually pass\ndata down the calling stack.\n\n**This is optional**. If you do not have Open Telemetry installed or\nsetup, functions here will not propagate the Pathway. You will need\nto manually propagate it in your application.\n\n[OTB]: https://github.com/open-telemetry/opentelemetry-erlang/blob/main/apps/opentelemetry_api/src/otel_baggage.erl","ref":"Datadog.DataStreams.Context.html","title":"Datadog.DataStreams.Context","type":"module"},{"doc":"Removes the current existing Pathway from OpenTelemetry. Returns\nthe value that existing in OpenTelemetry.","ref":"Datadog.DataStreams.Context.html#clear/0","title":"Datadog.DataStreams.Context.clear/0","type":"function"},{"doc":"Returns the current existing Pathway from OpenTelemetry. If\nthere is no Pathway in the current context, `nil` will be\nreturned","ref":"Datadog.DataStreams.Context.html#get/0","title":"Datadog.DataStreams.Context.get/0","type":"function"},{"doc":"Sets the given Pathway to the current Pathway in OpenTelemetry.","ref":"Datadog.DataStreams.Context.html#set/1","title":"Datadog.DataStreams.Context.set/1","type":"function"},{"doc":"Sets a checkpoint on the current existing Pathway. If one does\nnot exist, a new Pathway is created from the edge tags and\nreturned.","ref":"Datadog.DataStreams.Context.html#set_checkpoint/1","title":"Datadog.DataStreams.Context.set_checkpoint/1","type":"function"},{"doc":"Fowler-Noll-Vo variant 1 64-bit hash implementation. Use for\n`Datadog.DataStreams.Pathway` hashing.\n\nBased off the excellent asaaki work in the [`fnv` library][fnv].\n\n[fnv]: https://github.com/asaaki/fnv.ex","ref":"Datadog.DataStreams.FNV.html","title":"Datadog.DataStreams.FNV","type":"module"},{"doc":"Fowler-Noll-Vo variant 1 64-bit hash implementation.","ref":"Datadog.DataStreams.FNV.html#hash64/1","title":"Datadog.DataStreams.FNV.hash64/1","type":"function"},{"doc":"A pathway is used to monitor how payloads are sent across different services.\n\nAn example pathway would be:\n\n```text\nservice A -- edge 1 --> service B -- edge 2 --> service C\n```\n\nSo it's a bunch of services (we also call them \"nodes\") connected via edges.\nAs the payload is sent around, we save the start time (start of service A),\nand the start time of the previous service. This allows us to measure the\nlatency of each edge, as well as the latency from origin of any service.\n\nSee the [data-streams-go][DSG] package for more details.\n\n[DSG]: https://github.com/DataDog/data-streams-go/blob/main/datastreams/pathway.go","ref":"Datadog.DataStreams.Pathway.html","title":"Datadog.DataStreams.Pathway","type":"module"},{"doc":"Merges multiple pathways into one. The current implementation samples\none resulting pathway. A future implementation could be more clever\nand actually merge the pathways.","ref":"Datadog.DataStreams.Pathway.html#merge/1","title":"Datadog.DataStreams.Pathway.merge/1","type":"function"},{"doc":"Creates a new pathway struct.","ref":"Datadog.DataStreams.Pathway.html#new_pathway/1","title":"Datadog.DataStreams.Pathway.new_pathway/1","type":"function"},{"doc":"Creates a new pathway at a given time in unix epoch nanoseconds.","ref":"Datadog.DataStreams.Pathway.html#new_pathway/2","title":"Datadog.DataStreams.Pathway.new_pathway/2","type":"function"},{"doc":"Hashes all data for a pathway.","ref":"Datadog.DataStreams.Pathway.html#node_hash/4","title":"Datadog.DataStreams.Pathway.node_hash/4","type":"function"},{"doc":"iex> Pathway.node_hash(\"service-1\", \"env\", \"d:1\", [])\n    2071821778175304604\n\n    iex> # Invalid edge tag\n    ...> Pathway.node_hash(\"service-1\", \"env\", \"d:1\", [{\"edge\", \"1\"}])\n    2071821778175304604\n\n    iex> Pathway.node_hash(\"service-1\", \"env\", \"d:1\", [{\"type\", \"kafka\"}])\n    9272613839978655432","ref":"Datadog.DataStreams.Pathway.html#node_hash/4-examples","title":"Examples - Datadog.DataStreams.Pathway.node_hash/4","type":"function"},{"doc":"Hashes together a node and parent hash","ref":"Datadog.DataStreams.Pathway.html#pathway_hash/2","title":"Datadog.DataStreams.Pathway.pathway_hash/2","type":"function"},{"doc":"iex> Pathway.pathway_hash(0, 0)\n    9808874869469701221\n\n    iex> Pathway.pathway_hash(2071821778175304604, 0)\n    17210443572488294574\n\n    iex> Pathway.pathway_hash(0, 2071821778175304604)\n    12425197808660046030\n\n    iex> Pathway.pathway_hash(2071821778175304604, 17210443572488294574)\n    2003974475228685984","ref":"Datadog.DataStreams.Pathway.html#pathway_hash/2-examples","title":"Examples - Datadog.DataStreams.Pathway.pathway_hash/2","type":"function"},{"doc":"Sets a checkpoint on the pathway.","ref":"Datadog.DataStreams.Pathway.html#set_checkpoint/2","title":"Datadog.DataStreams.Pathway.set_checkpoint/2","type":"function"},{"doc":"Sets a checkpoint on the pathway at the given time in unix epoch nanoseconds.","ref":"Datadog.DataStreams.Pathway.html#set_checkpoint/3","title":"Datadog.DataStreams.Pathway.set_checkpoint/3","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Pathway.html#t:t/0","title":"Datadog.DataStreams.Pathway.t/0","type":"type"},{"doc":"Encoding logic for a payload. This is the top level struct we send\nto Datadog. It wraps all other information. These are primarily sent via\n[MessagePack][mp], although uses [Protobuf][pb] encoded binary for latency\nrecords.\n\n[mp]: https://msgpack.org/index.html\n[pb]: https://protobuf.dev/","ref":"Datadog.DataStreams.Payload.html","title":"Datadog.DataStreams.Payload","type":"module"},{"doc":"Adds an aggregator bucket to the payload.","ref":"Datadog.DataStreams.Payload.html#add_bucket/2","title":"Datadog.DataStreams.Payload.add_bucket/2","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Payload.html#add_bucket/3","title":"Datadog.DataStreams.Payload.add_bucket/3","type":"function"},{"doc":"Adds a map of buckets. This is the format the the aggregator uses internally.\nWe throw out the hash and just keep the buckets.","ref":"Datadog.DataStreams.Payload.html#add_buckets/3","title":"Datadog.DataStreams.Payload.add_buckets/3","type":"function"},{"doc":"Encodes the payload via MessagePack.","ref":"Datadog.DataStreams.Payload.html#encode/1","title":"Datadog.DataStreams.Payload.encode/1","type":"function"},{"doc":"Creates a new payload with the environment, service, and primary_tag\nfilled in from the `Datadog.DataStreams.Config` module.","ref":"Datadog.DataStreams.Payload.html#new/0","title":"Datadog.DataStreams.Payload.new/0","type":"function"},{"doc":"Returns how many stats are in the payload.","ref":"Datadog.DataStreams.Payload.html#stats_count/1","title":"Datadog.DataStreams.Payload.stats_count/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Payload.html#t:t/0","title":"Datadog.DataStreams.Payload.t/0","type":"type"},{"doc":"Handles propagating `Datadog.DataStreams.Pathway` via encoding and adding\nto message headers.","ref":"Datadog.DataStreams.Propagator.html","title":"Datadog.DataStreams.Propagator","type":"module"},{"doc":"Tries to decode a value into a pathway.","ref":"Datadog.DataStreams.Propagator.html#decode/1","title":"Datadog.DataStreams.Propagator.decode/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.decode( >)\n    %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000}\n\n    iex> Propagator.decode(\"invalid\")\n    nil","ref":"Datadog.DataStreams.Propagator.html#decode/1-examples","title":"Examples - Datadog.DataStreams.Propagator.decode/1","type":"function"},{"doc":"Decodes a pathway from a list or map of headers. If no matching header, or\nif the header is invalid, `nil` is returned.","ref":"Datadog.DataStreams.Propagator.html#decode_header/1","title":"Datadog.DataStreams.Propagator.decode_header/1","type":"function"},{"doc":"iex> Propagator.decode_header([{\"dd-pathway-ctx-base64\", \"rtARjT7H1+7gn/Cq02Hgn/Cq02E=\"}])\n    %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000}\n\n    iex> Propagator.decode_header(%{\"dd-pathway-ctx\" =>  >})\n    %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000}\n\n    iex> Propagator.decode_header(%{\"content-type\" => \"application-json\"})\n    nil","ref":"Datadog.DataStreams.Propagator.html#decode_header/1-examples","title":"Examples - Datadog.DataStreams.Propagator.decode_header/1","type":"function"},{"doc":"Tries to decode a Base64 encoded value into a pathway.","ref":"Datadog.DataStreams.Propagator.html#decode_str/1","title":"Datadog.DataStreams.Propagator.decode_str/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.decode_str(\"rtARjT7H1+7gn/Cq02Hgn/Cq02E=\")\n    %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000}\n\n    iex> Propagator.decode_str(\"invalid\")\n    nil","ref":"Datadog.DataStreams.Propagator.html#decode_str/1-examples","title":"Examples - Datadog.DataStreams.Propagator.decode_str/1","type":"function"},{"doc":"Decodes a pathway binary time from zigzag encoding.","ref":"Datadog.DataStreams.Propagator.html#decode_time/1","title":"Datadog.DataStreams.Propagator.decode_time/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.decode_time( >)\n    1677632342000000000\n\n    iex> Propagator.decode_time( >)\n    nil","ref":"Datadog.DataStreams.Propagator.html#decode_time/1-examples","title":"Examples - Datadog.DataStreams.Propagator.decode_time/1","type":"function"},{"doc":"Encodes a pathway to a string able to be placed in a header.","ref":"Datadog.DataStreams.Propagator.html#encode/1","title":"Datadog.DataStreams.Propagator.encode/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.encode(%Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000})\n     >\n\n    # Verified from golang implementation\n    iex> Propagator.encode(%Pathway{hash: 2003974475228685984, pathway_start: 1677628446000000000, edge_start: 1677628446000000000})\n     >","ref":"Datadog.DataStreams.Propagator.html#encode/1-examples","title":"Examples - Datadog.DataStreams.Propagator.encode/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Propagator.html#encode_header/1","title":"Datadog.DataStreams.Propagator.encode_header/1","type":"function"},{"doc":"Encodes a pathway into a list or map of headers.","ref":"Datadog.DataStreams.Propagator.html#encode_header/2","title":"Datadog.DataStreams.Propagator.encode_header/2","type":"function"},{"doc":"iex> Propagator.encode_header([], %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000})\n    [{\"dd-pathway-ctx\",  >}]\n\n    iex> Propagator.encode_header(%{}, %Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000})\n    %{\"dd-pathway-ctx\" =>  >}","ref":"Datadog.DataStreams.Propagator.html#encode_header/2-examples","title":"Examples - Datadog.DataStreams.Propagator.encode_header/2","type":"function"},{"doc":"Encodes a pathway to a string able to be placed in a header.","ref":"Datadog.DataStreams.Propagator.html#encode_str/1","title":"Datadog.DataStreams.Propagator.encode_str/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.encode_str(%Pathway{hash: 17210443572488294574, pathway_start: 1677632342000000000, edge_start: 1677632342000000000})\n    \"rtARjT7H1+7gn/Cq02Hgn/Cq02E=\"\n\n    # Verified from golang implementation\n    iex> Propagator.encode_str(%Pathway{hash: 2003974475228685984, pathway_start: 1677628446000000000, edge_start: 1677628446000000000})\n    \"oKb07iqMzxvg1JSn02Hg1JSn02E=\"","ref":"Datadog.DataStreams.Propagator.html#encode_str/1-examples","title":"Examples - Datadog.DataStreams.Propagator.encode_str/1","type":"function"},{"doc":"Encodes a pathway time using zigzag encoding.","ref":"Datadog.DataStreams.Propagator.html#encode_time/1","title":"Datadog.DataStreams.Propagator.encode_time/1","type":"function"},{"doc":"# Verified from golang implementation\n    iex> Propagator.encode_time(1677632342000000000)\n     >","ref":"Datadog.DataStreams.Propagator.html#encode_time/1-examples","title":"Examples - Datadog.DataStreams.Propagator.encode_time/1","type":"function"},{"doc":"Returns the well known header key for propagating encoded pathway data.","ref":"Datadog.DataStreams.Propagator.html#propagation_key/0","title":"Datadog.DataStreams.Propagator.propagation_key/0","type":"function"},{"doc":"iex> Propagator.propagation_key()\n    \"dd-pathway-ctx\"","ref":"Datadog.DataStreams.Propagator.html#propagation_key/0-examples","title":"Examples - Datadog.DataStreams.Propagator.propagation_key/0","type":"function"},{"doc":"Returns the well known base64 encoded header key for propagating encoded\npathway data.","ref":"Datadog.DataStreams.Propagator.html#propagation_key_base64/0","title":"Datadog.DataStreams.Propagator.propagation_key_base64/0","type":"function"},{"doc":"iex> Propagator.propagation_key_base64()\n    \"dd-pathway-ctx-base64\"","ref":"Datadog.DataStreams.Propagator.html#propagation_key_base64/0-examples","title":"Examples - Datadog.DataStreams.Propagator.propagation_key_base64/0","type":"function"},{"doc":"A helper module to enumerate and filter over data stream tags.","ref":"Datadog.DataStreams.Tags.html","title":"Datadog.DataStreams.Tags","type":"module"},{"doc":"Tags a list of tags and converts them to a list of binary tags. This is the\nformat that Datadog expects. This also orders the tags to ensure it matches\nother languages when hashing.","ref":"Datadog.DataStreams.Tags.html#encode/1","title":"Datadog.DataStreams.Tags.encode/1","type":"function"},{"doc":"iex> Tags.encode([{\"tag_two\", \"value_two\"}, {\"tag\", \"value\"}])\n    [\"tag:value\", \"tag_two:value_two\"]","ref":"Datadog.DataStreams.Tags.html#encode/1-examples","title":"Examples - Datadog.DataStreams.Tags.encode/1","type":"function"},{"doc":"Filters a list of tags to a known list of allowed tags. This varies based on\nthe context of where it is used.","ref":"Datadog.DataStreams.Tags.html#filter/2","title":"Datadog.DataStreams.Tags.filter/2","type":"function"},{"doc":"iex> [{\"key\", \"value\"}, {\"topic\", \"one\"}, {\"key_two\", \"value_two\"}]\n    ...> |> Tags.filter(:hash)\n    [{\"topic\", \"one\"}]","ref":"Datadog.DataStreams.Tags.html#filter/2-examples","title":"Examples - Datadog.DataStreams.Tags.filter/2","type":"function"},{"doc":"Parses any type of tag input and normalizes it to our internal `t:t` type.\nNote this will also filter out invalid data like nil values.","ref":"Datadog.DataStreams.Tags.html#parse/1","title":"Datadog.DataStreams.Tags.parse/1","type":"function"},{"doc":"iex> [{:key, \"value\"}, {:key_two, \"value_two\"}]\n    ...> |> Tags.parse()\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n\n    iex> [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n    ...> |> Tags.parse()\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n\n    iex> %{key: \"value\", key_two: \"value_two\"}\n    ...> |> Tags.parse()\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n\n    iex> %{\"key\" => \"value\", \"key_two\" => \"value_two\"}\n    ...> |> Tags.parse()\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n\n    iex> [\"key:value\", \"key_two:value_two\"]\n    ...> |> Tags.parse()\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n\n    iex> [{\"key\", nil}, {\"key_two\", \"value_two\"}]\n    ...> |> Tags.parse()\n    [{\"key_two\", \"value_two\"}]","ref":"Datadog.DataStreams.Tags.html#parse/1-examples","title":"Examples - Datadog.DataStreams.Tags.parse/1","type":"function"},{"doc":"An encoded list of tags. This is the format that Datadog expects when we send\ndata. It's a list of key value binary joined via `:`. For example:\n\"partition:1\", \"group:some-consumer-group\", \"topic:a-topic\".","ref":"Datadog.DataStreams.Tags.html#t:encoded/0","title":"Datadog.DataStreams.Tags.encoded/0","type":"type"},{"doc":"All allowed tag input types. This could be any one of the following examples:\n\n    [{:key, \"value\"}, {:key_two, \"value_two\"}]\n    [{\"key\", \"value\"}, {\"key_two\", \"value_two\"}]\n    %{key: \"value\", key_two: \"value_two\"}\n    %{\"key\" => \"value\", \"key_two\" => \"value_two\"}\n    [\"key:value\", \"key_two:value_two\"]","ref":"Datadog.DataStreams.Tags.html#t:input/0","title":"Datadog.DataStreams.Tags.input/0","type":"type"},{"doc":"The type all tags are internally mapped to.","ref":"Datadog.DataStreams.Tags.html#t:t/0","title":"Datadog.DataStreams.Tags.t/0","type":"type"},{"doc":"An HTTP client for Datadog data streams reporting. It uses the `Finch`\nlibrary for requests.","ref":"Datadog.DataStreams.Transport.html","title":"Datadog.DataStreams.Transport","type":"module"},{"doc":"Sends a MessagePack-ed binary to the Datadog agent. Ensuring it is\nacknowledged by response before returning `:ok`.","ref":"Datadog.DataStreams.Transport.html#send_pipeline_stats/1","title":"Datadog.DataStreams.Transport.send_pipeline_stats/1","type":"function"},{"doc":"","ref":"Datadog.DataStreams.Transport.html#t:response/0","title":"Datadog.DataStreams.Transport.response/0","type":"type"},{"doc":"A minimal implementation of the distributed quantile sketch (DDSketch)\nalgorithm as implemented in the [`sketches-go`][sg] library. For more\ninformation, please read the [`sketches-go` README][sg].\n\nThis is a minimal implementation needed to support DataDog data streams.\nSome initial work was taken from the great [`dog_sketch` Elixir library][dd].\nThis library includes some DataDog specific work like Protobuf encoding. It's\nworth mentioning that some of this code looks out of place in Elixir. That is\nbecause it's pulled directly from the [`sketches-go`][sg] library and kept\nsimilar for ease of debugging and backporting fixes.\n\n[sg]: https://github.com/DataDog/sketches-go\n[dd]: https://github.com/moosecodebv/dog_sketch","ref":"Datadog.Sketch.html","title":"Datadog.Sketch","type":"module"},{"doc":"Adds a value to the sketch.","ref":"Datadog.Sketch.html#add/2","title":"Datadog.Sketch.add/2","type":"function"},{"doc":"iex> %Sketch{} = Sketch.add(Sketch.new_default(), 4293.7)\n\n    iex> %Sketch{} = Sketch.add(Sketch.new_default(), -4592.3)\n\n    iex> %Sketch{} = Sketch.add(Sketch.new_default(), 0.0)","ref":"Datadog.Sketch.html#add/2-examples","title":"Examples - Datadog.Sketch.add/2","type":"function"},{"doc":"Adds a bin type to the sketch.","ref":"Datadog.Sketch.html#add_bin/2","title":"Datadog.Sketch.add_bin/2","type":"function"},{"doc":"iex> %Sketch{} = Sketch.add_bin(Sketch.new_default(), %{index: 100, count: 13.13})","ref":"Datadog.Sketch.html#add_bin/2-examples","title":"Examples - Datadog.Sketch.add_bin/2","type":"function"},{"doc":"Adds multiple bin types to the sketch.","ref":"Datadog.Sketch.html#add_bins/2","title":"Datadog.Sketch.add_bins/2","type":"function"},{"doc":"iex> %Sketch{} = Sketch.add_bins(Sketch.new_default(), [\n      ...>   %{index: 100, count: 13.13},\n      ...>   %{index: 20, count: 2342.4}\n      ...> ])","ref":"Datadog.Sketch.html#add_bins/2-examples","title":"Examples - Datadog.Sketch.add_bins/2","type":"function"},{"doc":"Adds a value to the sketch with a float count.","ref":"Datadog.Sketch.html#add_with_count/3","title":"Datadog.Sketch.add_with_count/3","type":"function"},{"doc":"iex> %Sketch{} = Sketch.add_with_count(Sketch.new_default(), 4293.7, 294)\n\n    iex> %Sketch{} = Sketch.add_with_count(Sketch.new_default(), -4592.3, 23)\n\n    iex> %Sketch{} = Sketch.add_with_count(Sketch.new_default(), 0.0, 10)","ref":"Datadog.Sketch.html#add_with_count/3-examples","title":"Examples - Datadog.Sketch.add_with_count/3","type":"function"},{"doc":"Returns true if no value has been added to this sketch.","ref":"Datadog.Sketch.html#empty?/1","title":"Datadog.Sketch.empty?/1","type":"function"},{"doc":"iex> Sketch.empty?(Sketch.new_default())\n    true\n\n    iex> Sketch.new_default()\n    ...> |> Sketch.add_with_count(42, 482.23)\n    ...> |> Sketch.empty?()\n    false\n\n    iex> Sketch.new_default()\n    ...> |> Sketch.add_with_count(-75, 157)\n    ...> |> Sketch.empty?()\n    false","ref":"Datadog.Sketch.html#empty?/1-examples","title":"Examples - Datadog.Sketch.empty?/1","type":"function"},{"doc":"Return the total number of values that have been added to this sketch.","ref":"Datadog.Sketch.html#get_count/1","title":"Datadog.Sketch.get_count/1","type":"function"},{"doc":"iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 423.43},\n    ...>   %{index: 244, count: 1238.123},\n    ...>   %{index: 124, count: 2184.124}\n    ...> ])\n    ...> Sketch.get_count(sketch)\n    3845.6769999999997","ref":"Datadog.Sketch.html#get_count/1-examples","title":"Examples - Datadog.Sketch.get_count/1","type":"function"},{"doc":"Returns the maximum value that has been added to this sketch.","ref":"Datadog.Sketch.html#get_max_value/1","title":"Datadog.Sketch.get_max_value/1","type":"function"},{"doc":"iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 34},\n    ...>   %{index: -423, count: 571},\n    ...>   %{index: 0, count: 27.1}\n    ...> ])\n    ...> Sketch.get_max_value(sketch)\n    12.061674179039226","ref":"Datadog.Sketch.html#get_max_value/1-examples","title":"Examples - Datadog.Sketch.get_max_value/1","type":"function"},{"doc":"Returns the minimum value that has been added to this sketch.","ref":"Datadog.Sketch.html#get_min_value/1","title":"Datadog.Sketch.get_min_value/1","type":"function"},{"doc":"iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 34},\n    ...>   %{index: -423, count: 571},\n    ...>   %{index: 0, count: 27.1}\n    ...> ])\n    ...> Sketch.get_min_value(sketch)\n    -424.1773628048435","ref":"Datadog.Sketch.html#get_min_value/1-examples","title":"Examples - Datadog.Sketch.get_min_value/1","type":"function"},{"doc":"Returns an approximation of the sum of the values that have been added\nto the sketch. If the values that have been added to the sketch all\nhave the same sign, the approximation error has the relative accuracy\nguarantees of the mapping used for this sketch.","ref":"Datadog.Sketch.html#get_sum/1","title":"Datadog.Sketch.get_sum/1","type":"function"},{"doc":"# Verified with golang implementation (within accuracy)\n    iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 34.0},\n    ...>   %{index: -24, count: 84.0},\n    ...>   %{index: 0, count: 2.4}\n    ...> ])\n    ...> Sketch.get_sum(sketch)\n    -1589.8430984284082","ref":"Datadog.Sketch.html#get_sum/1-examples","title":"Examples - Datadog.Sketch.get_sum/1","type":"function"},{"doc":"Return the value at the specified quantile.","ref":"Datadog.Sketch.html#get_value_at_quantile/2","title":"Datadog.Sketch.get_value_at_quantile/2","type":"function"},{"doc":"# Validated with golang implementation\n    iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 423.43},\n    ...>   %{index: 244, count: 1238.123},\n    ...>   %{index: 124, count: 2184.124}\n    ...> ])\n    ...> Sketch.get_value_at_quantile(sketch, 0.4)\n    125.2248607394614","ref":"Datadog.Sketch.html#get_value_at_quantile/2-examples","title":"Examples - Datadog.Sketch.get_value_at_quantile/2","type":"function"},{"doc":"Return the values at the respective specified quantiles.","ref":"Datadog.Sketch.html#get_values_at_quantiles/2","title":"Datadog.Sketch.get_values_at_quantiles/2","type":"function"},{"doc":"# Validated with golang implementation (within accuracy)\n    iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 423.43},\n    ...>   %{index: 244, count: 1238.123},\n    ...>   %{index: 124, count: 2184.124}\n    ...> ])\n    ...> Sketch.get_values_at_quantiles(sketch, [0.1, 0.25, 0.5])\n    [12.061674179039226, 125.2248607394614, 125.2248607394614]","ref":"Datadog.Sketch.html#get_values_at_quantiles/2-examples","title":"Examples - Datadog.Sketch.get_values_at_quantiles/2","type":"function"},{"doc":"Returns the number of zero values that have been added to this sketch.","ref":"Datadog.Sketch.html#get_zero_count/1","title":"Datadog.Sketch.get_zero_count/1","type":"function"},{"doc":"iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n    ...>   %{index: 12, count: 34},\n    ...>   %{index: -423, count: 571},\n    ...>   %{index: 0, count: 27.1}\n    ...> ])\n    ...> Sketch.get_zero_count(sketch)\n    27.1","ref":"Datadog.Sketch.html#get_zero_count/1-examples","title":"Examples - Datadog.Sketch.get_zero_count/1","type":"function"},{"doc":"Creates a new Sketch.","ref":"Datadog.Sketch.html#new/2","title":"Datadog.Sketch.new/2","type":"function"},{"doc":"iex> index_mapping = Sketch.IndexMapping.Logarithmic.new(0.01)\n    ...> store = Sketch.Store.Dense.new()\n    ...> %Sketch{} = Sketch.new(index_mapping, store)","ref":"Datadog.Sketch.html#new/2-examples","title":"Examples - Datadog.Sketch.new/2","type":"function"},{"doc":"Creates a new Sketch with separate stores for positive and negative\nvalues.","ref":"Datadog.Sketch.html#new/3","title":"Datadog.Sketch.new/3","type":"function"},{"doc":"iex> index_mapping = Sketch.IndexMapping.Logarithmic.new(0.01)\n    ...> positive_store = Sketch.Store.Dense.new()\n    ...> negative_store = Sketch.Store.Dense.new()\n    ...> %Sketch{} = Sketch.new(index_mapping, positive_store, negative_store)","ref":"Datadog.Sketch.html#new/3-examples","title":"Examples - Datadog.Sketch.new/3","type":"function"},{"doc":"Creates a new Sketch with the default index mapping and store values.\nThis equates to using the `IndexMapping.Logarithmic` module with a\n`0.01` relative accuracy and the `Store.Dense` store.","ref":"Datadog.Sketch.html#new_default/0","title":"Datadog.Sketch.new_default/0","type":"function"},{"doc":"iex> %Sketch{} = Sketch.new_default()","ref":"Datadog.Sketch.html#new_default/0-examples","title":"Examples - Datadog.Sketch.new_default/0","type":"function"},{"doc":"Reweight multiples all values from the sketch by `weight`, but keeps\nthe same global distribution. `weight` has to be strictly greater\nthan zero.","ref":"Datadog.Sketch.html#reweight/2","title":"Datadog.Sketch.reweight/2","type":"function"},{"doc":"# Verified with golang implementation (within accuracy)\n      iex> sketch = Sketch.add_bins(Sketch.new_default(), [\n      ...>   %{index: -7, count: 10.0},\n      ...>   %{index: 24, count: 20.0},\n      ...>   %{index: 3, count: 30.0}\n      ...> ])\n      ...> sketch = Sketch.reweight(sketch, 2.5)\n      ...> Sketch.get_sum(sketch)\n      1237.7881696246109","ref":"Datadog.Sketch.html#reweight/2-examples","title":"Examples - Datadog.Sketch.reweight/2","type":"function"},{"doc":"Returns a Protobuf-able struct for the sketch. Used for sending data to\nDatadog.","ref":"Datadog.Sketch.html#to_proto/1","title":"Datadog.Sketch.to_proto/1","type":"function"},{"doc":"iex> %Sketch.Protobuf.DDSketch{} = Sketch.to_proto(Sketch.new_default())","ref":"Datadog.Sketch.html#to_proto/1-examples","title":"Examples - Datadog.Sketch.to_proto/1","type":"function"},{"doc":"","ref":"Datadog.Sketch.html#t:t/0","title":"Datadog.Sketch.t/0","type":"type"},{"doc":"Basic module for handling various index mapping algorithms. All functions in\nthis module proxy to the respective index mapping implementation module.","ref":"Datadog.Sketch.IndexMapping.html","title":"Datadog.Sketch.IndexMapping","type":"behaviour"},{"doc":"Checks if an index mapping matches another index mapping.\n\n    iex> implementation_one = IndexMapping.Logarithmic.new(0.0000000000001)\n    ...> implementation_two = IndexMapping.Logarithmic.new(0.0000000000002)\n    ...> IndexMapping.equals(implementation_one, implementation_two)\n    true\n\n    iex> implementation_one = IndexMapping.Logarithmic.new(0.01)\n    ...> implementation_two = IndexMapping.Logarithmic.new(0.00001)\n    ...> IndexMapping.equals(implementation_one, implementation_two)\n    false","ref":"Datadog.Sketch.IndexMapping.html#equals/2","title":"Datadog.Sketch.IndexMapping.equals/2","type":"function"},{"doc":"Checks if an index mapping matches another index mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:equals/2","title":"Datadog.Sketch.IndexMapping.equals/2","type":"callback"},{"doc":"Returns value after mapping.\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.index(index_mapping, 115)\n    237\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.001)\n    ...> IndexMapping.index(index_mapping, 12345678901234567890)\n    21979","ref":"Datadog.Sketch.IndexMapping.html#index/2","title":"Datadog.Sketch.IndexMapping.index/2","type":"function"},{"doc":"Returns value after mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:index/2","title":"Datadog.Sketch.IndexMapping.index/2","type":"callback"},{"doc":"Returns the lower bound the mapping can contain.\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.lower_bound(index_mapping, 0)\n    1.0\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.lower_bound(index_mapping, 10)\n    1.2214109013609646","ref":"Datadog.Sketch.IndexMapping.html#lower_bound/2","title":"Datadog.Sketch.IndexMapping.lower_bound/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain.","ref":"Datadog.Sketch.IndexMapping.html#c:lower_bound/2","title":"Datadog.Sketch.IndexMapping.lower_bound/2","type":"callback"},{"doc":"Returns the lower bound the mapping can contain.\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.relative_accuracy(index_mapping)\n    0.009999999999999898","ref":"Datadog.Sketch.IndexMapping.html#relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.relative_accuracy/1","type":"function"},{"doc":"Returns the relative accuracy of the mapping.","ref":"Datadog.Sketch.IndexMapping.html#c:relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.relative_accuracy/1","type":"callback"},{"doc":"Returns a Protobuf-able struct for the index mapping. Used for\nsending data to Datadog.\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.to_proto(index_mapping)\n    %Datadog.Sketch.Protobuf.IndexMapping{gamma: 1.02020202020202, interpolation: :NONE}","ref":"Datadog.Sketch.IndexMapping.html#to_proto/1","title":"Datadog.Sketch.IndexMapping.to_proto/1","type":"function"},{"doc":"Returns a Protobuf-able struct for the index mapping. Used for\nsending data to Datadog.","ref":"Datadog.Sketch.IndexMapping.html#c:to_proto/1","title":"Datadog.Sketch.IndexMapping.to_proto/1","type":"callback"},{"doc":"Takes a mapped value and returns the original value within the set accuracy.\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.01)\n    ...> IndexMapping.value(index_mapping, 237)\n    115.59680764552533\n\n    iex> index_mapping = IndexMapping.Logarithmic.new(0.001)\n    ...> IndexMapping.value(index_mapping, 21979)\n    1.23355147396003e19","ref":"Datadog.Sketch.IndexMapping.html#value/2","title":"Datadog.Sketch.IndexMapping.value/2","type":"function"},{"doc":"Takes a mapped value and returns the original value within the set accuracy.","ref":"Datadog.Sketch.IndexMapping.html#c:value/2","title":"Datadog.Sketch.IndexMapping.value/2","type":"callback"},{"doc":"Checks if the two values are within the tolerance given.","ref":"Datadog.Sketch.IndexMapping.html#within_tolerance/3","title":"Datadog.Sketch.IndexMapping.within_tolerance/3","type":"function"},{"doc":"iex> IndexMapping.within_tolerance(90, 134, 50)\n    true\n\n    iex> IndexMapping.within_tolerance(0.00128, 0.00864, 0.01)\n    false","ref":"Datadog.Sketch.IndexMapping.html#within_tolerance/3-examples","title":"Examples - Datadog.Sketch.IndexMapping.within_tolerance/3","type":"function"},{"doc":"A general struct that has index mapping data. Every index mapping\nimplementation _must_ contain this data.","ref":"Datadog.Sketch.IndexMapping.html#t:t/0","title":"Datadog.Sketch.IndexMapping.t/0","type":"type"},{"doc":"LogarithmicMapping is an IndexMapping that is memory-optimal, that is to say\nthat given a targeted relative accuracy, it requires the least number of\nindices to cover a given range of values. This is done by logarithmically\nmapping floating-point values to integers.\n\nNote, since Erlang (and therefor Elixir) do math pretty differently than\ngolang, this module does not contain a min or max indexable value. This\nfollows in line with the Erlang philosophy of \"let it crash\" and simplifies\nour logic handling.","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html","title":"Datadog.Sketch.IndexMapping.Logarithmic","type":"module"},{"doc":"Checks if an index mapping matches another index mapping.\n\n    iex> implementation_one = Logarithmic.new(0.0000000000001)\n    ...> implementation_two = Logarithmic.new(0.0000000000002)\n    ...> Logarithmic.equals(implementation_one, implementation_two)\n    true\n\n    iex> implementation_one = Logarithmic.new(0.01)\n    ...> implementation_two = Logarithmic.new(0.00001)\n    ...> Logarithmic.equals(implementation_one, implementation_two)\n    false","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#equals/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.equals/2","type":"function"},{"doc":"Returns value after mapping via logarithmic equation.\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.index(index_mapping, 115)\n    237\n\n    iex> index_mapping = Logarithmic.new(0.001)\n    ...> Logarithmic.index(index_mapping, 12345678901234567890)\n    21979","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#index/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.index/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain.\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.lower_bound(index_mapping, 0)\n    1.0\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.lower_bound(index_mapping, 10)\n    1.2214109013609646","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#lower_bound/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.lower_bound/2","type":"function"},{"doc":"Creates a new Logarithmic index mapping with the given accuracy.","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#new/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.new/1","type":"function"},{"doc":"iex> Logarithmic.new(-1)\n    ** (ArgumentError) The relative accuracy must be between 0, and 1.\n\n    iex> Logarithmic.new(0.01)\n    %Logarithmic{gamma: 1.02020202020202, multiplier: 49.99833328888678}\n\n    iex> Logarithmic.new(0.0001)\n    %Logarithmic{gamma: 1.0002000200020003, multiplier: 4999.999983331928}","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#new/1-examples","title":"Examples - Datadog.Sketch.IndexMapping.Logarithmic.new/1","type":"function"},{"doc":"Creates a new Logarithmic index mapping with the given gamma\nand index offset.\n\n    iex> Logarithmic.new(0.5, 0.0)\n    ** (ArgumentError) Gamma must be greater than 1.\n\n    iex> Logarithmic.new(1.0002000200020003, 0.0)\n    %Logarithmic{gamma: 1.0002000200020003, multiplier: 4999.999983331928}\n\n    iex> Logarithmic.new(1.0002000200020003, 1.5)\n    %Logarithmic{gamma: 1.0002000200020003, index_offset: 1.5, multiplier: 4999.999983331928}","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#new/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.new/2","type":"function"},{"doc":"Returns the lower bound the mapping can contain.\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.relative_accuracy(index_mapping)\n    0.009999999999999898","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#relative_accuracy/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.relative_accuracy/1","type":"function"},{"doc":"Returns a Protobuf-able struct for the index mapping., Used for\nsending data to Datadog.\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.to_proto(index_mapping)\n    %Datadog.Sketch.Protobuf.IndexMapping{gamma: 1.02020202020202, interpolation: :NONE}","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#to_proto/1","title":"Datadog.Sketch.IndexMapping.Logarithmic.to_proto/1","type":"function"},{"doc":"Takes a mapped value and returns the original value within the set accuracy.\n\n    iex> index_mapping = Logarithmic.new(0.01)\n    ...> Logarithmic.value(index_mapping, 237)\n    115.59680764552533\n\n    iex> index_mapping = Logarithmic.new(0.001)\n    ...> Logarithmic.value(index_mapping, 21979)\n    1.23355147396003e19","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#value/2","title":"Datadog.Sketch.IndexMapping.Logarithmic.value/2","type":"function"},{"doc":"","ref":"Datadog.Sketch.IndexMapping.Logarithmic.html#t:t/0","title":"Datadog.Sketch.IndexMapping.Logarithmic.t/0","type":"type"},{"doc":"Different stores use different data structures and techniques to store data.\nEach with unique trade offs and memory usage. All functions in this module\nproxy to the respective index mapping implementation module.\n\nAll stores implement the `Enumerable` protocol to iterate over all stored\ndata easily.","ref":"Datadog.Sketch.Store.html","title":"Datadog.Sketch.Store","type":"behaviour"},{"doc":"Adds a number to the store.","ref":"Datadog.Sketch.Store.html#add/2","title":"Datadog.Sketch.Store.add/2","type":"function"},{"doc":"iex> %Store.Dense{} = Store.add(Store.Dense.new(), 100)","ref":"Datadog.Sketch.Store.html#add/2-examples","title":"Examples - Datadog.Sketch.Store.add/2","type":"function"},{"doc":"Adds a number to the store.","ref":"Datadog.Sketch.Store.html#c:add/2","title":"Datadog.Sketch.Store.add/2","type":"callback"},{"doc":"Adds a bin type to the store.","ref":"Datadog.Sketch.Store.html#add_bin/2","title":"Datadog.Sketch.Store.add_bin/2","type":"function"},{"doc":"iex> %Store.Dense{} = Store.add_bin(Store.Dense.new(), %{index: 100, count: 13.13})","ref":"Datadog.Sketch.Store.html#add_bin/2-examples","title":"Examples - Datadog.Sketch.Store.add_bin/2","type":"function"},{"doc":"Adds a bin type to the store.","ref":"Datadog.Sketch.Store.html#c:add_bin/2","title":"Datadog.Sketch.Store.add_bin/2","type":"callback"},{"doc":"Adds multiple bin types to the store.","ref":"Datadog.Sketch.Store.html#add_bins/2","title":"Datadog.Sketch.Store.add_bins/2","type":"function"},{"doc":"iex> %Store.Dense{} = Store.add_bins(Store.Dense.new(), [\n      ...>   %{index: 100, count: 13.13},\n      ...>   %{index: 20, count: 2342.4}\n      ...> ])","ref":"Datadog.Sketch.Store.html#add_bins/2-examples","title":"Examples - Datadog.Sketch.Store.add_bins/2","type":"function"},{"doc":"Adds multiple bin types to the store.","ref":"Datadog.Sketch.Store.html#c:add_bins/2","title":"Datadog.Sketch.Store.add_bins/2","type":"callback"},{"doc":"Adds a number to the store `count` number of times.","ref":"Datadog.Sketch.Store.html#add_with_count/3","title":"Datadog.Sketch.Store.add_with_count/3","type":"function"},{"doc":"iex> %Store.Dense{} = Store.add_with_count(Store.Dense.new(), 100, 13.13)\n\n    iex> %Store.Dense{} = Store.add_with_count(Store.Dense.new(), 987, 8.3e12)","ref":"Datadog.Sketch.Store.html#add_with_count/3-examples","title":"Examples - Datadog.Sketch.Store.add_with_count/3","type":"function"},{"doc":"Adds a number to the store `count` number of times.","ref":"Datadog.Sketch.Store.html#c:add_with_count/3","title":"Datadog.Sketch.Store.add_with_count/3","type":"callback"},{"doc":"Checks if the store has any information in it.","ref":"Datadog.Sketch.Store.html#empty?/1","title":"Datadog.Sketch.Store.empty?/1","type":"function"},{"doc":"iex> store = Store.Dense.new()\n    ...> Store.empty?(store)\n    true\n\n    iex> store = Store.add(Store.Dense.new(), 754)\n    ...> Store.empty?(store)\n    false","ref":"Datadog.Sketch.Store.html#empty?/1-examples","title":"Examples - Datadog.Sketch.Store.empty?/1","type":"function"},{"doc":"Checks if the store has any information in it.","ref":"Datadog.Sketch.Store.html#c:empty?/1","title":"Datadog.Sketch.Store.empty?/1","type":"callback"},{"doc":"Return the key for the value at rank.","ref":"Datadog.Sketch.Store.html#key_at_rank/2","title":"Datadog.Sketch.Store.key_at_rank/2","type":"function"},{"doc":"iex> store = Store.add(Store.Dense.new(), 128)\n      ...> Store.key_at_rank(store, 0.0)\n      128","ref":"Datadog.Sketch.Store.html#key_at_rank/2-examples","title":"Examples - Datadog.Sketch.Store.key_at_rank/2","type":"function"},{"doc":"Return the key for the value at rank.","ref":"Datadog.Sketch.Store.html#c:key_at_rank/2","title":"Datadog.Sketch.Store.key_at_rank/2","type":"callback"},{"doc":"Returns the maximum index of the store.","ref":"Datadog.Sketch.Store.html#max_index/1","title":"Datadog.Sketch.Store.max_index/1","type":"function"},{"doc":"iex> store = Store.Dense.new()\n      ...> Store.max_index(store)\n      0","ref":"Datadog.Sketch.Store.html#max_index/1-examples","title":"Examples - Datadog.Sketch.Store.max_index/1","type":"function"},{"doc":"Returns the maximum index of the store.","ref":"Datadog.Sketch.Store.html#c:max_index/1","title":"Datadog.Sketch.Store.max_index/1","type":"callback"},{"doc":"Returns the minimum index of the store.","ref":"Datadog.Sketch.Store.html#min_index/1","title":"Datadog.Sketch.Store.min_index/1","type":"function"},{"doc":"iex> store = Store.Dense.new()\n    ...> Store.min_index(store)\n    0","ref":"Datadog.Sketch.Store.html#min_index/1-examples","title":"Examples - Datadog.Sketch.Store.min_index/1","type":"function"},{"doc":"Returns the minimum index of the store.","ref":"Datadog.Sketch.Store.html#c:min_index/1","title":"Datadog.Sketch.Store.min_index/1","type":"callback"},{"doc":"Maps over all values and multiplies by the given weight.","ref":"Datadog.Sketch.Store.html#reweight/2","title":"Datadog.Sketch.Store.reweight/2","type":"function"},{"doc":"iex> store = Store.add_bins(Store.Dense.new(), [\n      ...>   %{index: 4, count: 10.0},\n      ...>   %{index: 2, count: 20.0},\n      ...>   %{index: 6, count: 30.0}\n      ...> ])\n      ...> store = Store.reweight(store, 2)\n      ...> Store.total_count(store)\n      120.0","ref":"Datadog.Sketch.Store.html#reweight/2-examples","title":"Examples - Datadog.Sketch.Store.reweight/2","type":"function"},{"doc":"Maps over all values and multiplies by the given weight.","ref":"Datadog.Sketch.Store.html#c:reweight/2","title":"Datadog.Sketch.Store.reweight/2","type":"callback"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to\nDatadog.","ref":"Datadog.Sketch.Store.html#to_proto/1","title":"Datadog.Sketch.Store.to_proto/1","type":"function"},{"doc":"iex> %Datadog.Sketch.Protobuf.Store{} = Store.to_proto(Store.Dense.new())","ref":"Datadog.Sketch.Store.html#to_proto/1-examples","title":"Examples - Datadog.Sketch.Store.to_proto/1","type":"function"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to\nDatadog.","ref":"Datadog.Sketch.Store.html#c:to_proto/1","title":"Datadog.Sketch.Store.to_proto/1","type":"callback"},{"doc":"Returns the total amount of counts stored.","ref":"Datadog.Sketch.Store.html#total_count/1","title":"Datadog.Sketch.Store.total_count/1","type":"function"},{"doc":"iex> store = Store.Dense.new()\n    ...> Store.total_count(store)\n    0.0\n\n    iex> store = Store.add_with_count(Store.Dense.new(), 754, 42.42)\n    ...> Store.total_count(store)\n    42.42","ref":"Datadog.Sketch.Store.html#total_count/1-examples","title":"Examples - Datadog.Sketch.Store.total_count/1","type":"function"},{"doc":"Returns the total amount of counts stored.","ref":"Datadog.Sketch.Store.html#c:total_count/1","title":"Datadog.Sketch.Store.total_count/1","type":"callback"},{"doc":"A concise way to specify an index and count in one arg.","ref":"Datadog.Sketch.Store.html#t:bin/0","title":"Datadog.Sketch.Store.bin/0","type":"type"},{"doc":"A basic struct. This is used to keep tight type definitions.\nEach store implements this struct differently.","ref":"Datadog.Sketch.Store.html#t:t/0","title":"Datadog.Sketch.Store.t/0","type":"type"},{"doc":"The Dense store is a dynamically growing contiguous (non-sparse) store. The\nnumber of bins are bound only by the size of the `:array` that can be allocated.","ref":"Datadog.Sketch.Store.Dense.html","title":"Datadog.Sketch.Store.Dense","type":"module"},{"doc":"Adds a number to the store.","ref":"Datadog.Sketch.Store.Dense.html#add/2","title":"Datadog.Sketch.Store.Dense.add/2","type":"function"},{"doc":"iex> %Dense{} = Dense.add(Dense.new(), 100)","ref":"Datadog.Sketch.Store.Dense.html#add/2-examples","title":"Examples - Datadog.Sketch.Store.Dense.add/2","type":"function"},{"doc":"Adds a bin type to the store.","ref":"Datadog.Sketch.Store.Dense.html#add_bin/2","title":"Datadog.Sketch.Store.Dense.add_bin/2","type":"function"},{"doc":"iex> %Dense{} = Dense.add_bin(Dense.new(), %{index: 100, count: 13.13})","ref":"Datadog.Sketch.Store.Dense.html#add_bin/2-examples","title":"Examples - Datadog.Sketch.Store.Dense.add_bin/2","type":"function"},{"doc":"Adds multiple bin types to the store.","ref":"Datadog.Sketch.Store.Dense.html#add_bins/2","title":"Datadog.Sketch.Store.Dense.add_bins/2","type":"function"},{"doc":"iex> %Dense{} = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 100, count: 13.13},\n      ...>   %{index: 20, count: 2342.4}\n      ...> ])","ref":"Datadog.Sketch.Store.Dense.html#add_bins/2-examples","title":"Examples - Datadog.Sketch.Store.Dense.add_bins/2","type":"function"},{"doc":"Adds a number to the store `count` number of times.","ref":"Datadog.Sketch.Store.Dense.html#add_with_count/3","title":"Datadog.Sketch.Store.Dense.add_with_count/3","type":"function"},{"doc":"iex> %Dense{} = Dense.add_with_count(Dense.new(), 100, 13.13)\n\n      iex> %Dense{} = Dense.add_with_count(Dense.new(), 987, 8.3e12)","ref":"Datadog.Sketch.Store.Dense.html#add_with_count/3-examples","title":"Examples - Datadog.Sketch.Store.Dense.add_with_count/3","type":"function"},{"doc":"Checks if the store has any information in it.","ref":"Datadog.Sketch.Store.Dense.html#empty?/1","title":"Datadog.Sketch.Store.Dense.empty?/1","type":"function"},{"doc":"iex> store = Dense.new()\n      ...> Dense.empty?(store)\n      true\n\n      iex> store = Dense.add(Dense.new(), 754)\n      ...> Dense.empty?(store)\n      false","ref":"Datadog.Sketch.Store.Dense.html#empty?/1-examples","title":"Examples - Datadog.Sketch.Store.Dense.empty?/1","type":"function"},{"doc":"Return the key for the value at rank.","ref":"Datadog.Sketch.Store.Dense.html#key_at_rank/2","title":"Datadog.Sketch.Store.Dense.key_at_rank/2","type":"function"},{"doc":"# Matches the golang implementation when no data is present.\n      iex> store = Dense.new()\n      ...> Dense.key_at_rank(store, 0.0)\n      -134_217_729\n\n      # Verified matching to golang implementation\n      iex> store = Dense.add(Dense.new(), 128)\n      ...> Dense.key_at_rank(store, 0.0)\n      128\n\n      # Verified matching to golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 12, count: 423.43},\n      ...>   %{index: 244, count: 1238.123},\n      ...>   %{index: 124, count: 2184.124}\n      ...> ])\n      ...> Dense.key_at_rank(store, 64.0)\n      12","ref":"Datadog.Sketch.Store.Dense.html#key_at_rank/2-examples","title":"Examples - Datadog.Sketch.Store.Dense.key_at_rank/2","type":"function"},{"doc":"Returns the maximum index of the store.","ref":"Datadog.Sketch.Store.Dense.html#max_index/1","title":"Datadog.Sketch.Store.Dense.max_index/1","type":"function"},{"doc":"iex> store = Dense.new()\n      ...> Dense.max_index(store)\n      0\n\n      iex> store = Dense.add(Dense.new(), 128)\n      ...> Dense.max_index(store)\n      128\n\n      # Verified against golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 4, count: 12.48},\n      ...>   %{index: 65, count: 12.48},\n      ...>   %{index: 37, count: 847.4}\n      ...> ])\n      ...> Dense.max_index(store)\n      65","ref":"Datadog.Sketch.Store.Dense.html#max_index/1-examples","title":"Examples - Datadog.Sketch.Store.Dense.max_index/1","type":"function"},{"doc":"Returns the minimum index of the store.","ref":"Datadog.Sketch.Store.Dense.html#min_index/1","title":"Datadog.Sketch.Store.Dense.min_index/1","type":"function"},{"doc":"iex> store = Dense.new()\n      ...> Dense.min_index(store)\n      0\n\n      # Verified against golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 4, count: 12.48},\n      ...>   %{index: 65, count: 12.48},\n      ...>   %{index: 37, count: 847.4}\n      ...> ])\n      ...> Dense.min_index(store)\n      4","ref":"Datadog.Sketch.Store.Dense.html#min_index/1-examples","title":"Examples - Datadog.Sketch.Store.Dense.min_index/1","type":"function"},{"doc":"Creates a new dense store.","ref":"Datadog.Sketch.Store.Dense.html#new/0","title":"Datadog.Sketch.Store.Dense.new/0","type":"function"},{"doc":"iex> Dense.new()\n      %Dense{}","ref":"Datadog.Sketch.Store.Dense.html#new/0-examples","title":"Examples - Datadog.Sketch.Store.Dense.new/0","type":"function"},{"doc":"Maps over all values and multiplies by the given weight.","ref":"Datadog.Sketch.Store.Dense.html#reweight/2","title":"Datadog.Sketch.Store.Dense.reweight/2","type":"function"},{"doc":"# Verified matching to golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 4, count: 10.0},\n      ...>   %{index: 2, count: 20.0},\n      ...>   %{index: 6, count: 30.0}\n      ...> ])\n      ...> store = Dense.reweight(store, 2)\n      ...> Dense.total_count(store)\n      120.0","ref":"Datadog.Sketch.Store.Dense.html#reweight/2-examples","title":"Examples - Datadog.Sketch.Store.Dense.reweight/2","type":"function"},{"doc":"Returns a struct for Protobuf encoding. Used for sending data to\nDatadog.","ref":"Datadog.Sketch.Store.Dense.html#to_proto/1","title":"Datadog.Sketch.Store.Dense.to_proto/1","type":"function"},{"doc":"# Verified matching to golang implementation\n      iex> store = Dense.add(Dense.new(), 4)\n      ...> Dense.to_proto(store)\n      %Datadog.Sketch.Protobuf.Store{\n        binCounts: %{},\n        contiguousBinCounts: [1.0],\n        contiguousBinIndexOffset: 4\n      }\n\n      # Verified matching to golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 4, count: 12.48},\n      ...>   %{index: 65, count: 12.48},\n      ...>   %{index: 37, count: 847.4}\n      ...> ])\n      ...> Dense.to_proto(store)\n      %Datadog.Sketch.Protobuf.Store{\n        binCounts: %{},\n        contiguousBinCounts: [12.48, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 847.4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.48],\n        contiguousBinIndexOffset: 4\n      }","ref":"Datadog.Sketch.Store.Dense.html#to_proto/1-examples","title":"Examples - Datadog.Sketch.Store.Dense.to_proto/1","type":"function"},{"doc":"Returns the total amount of counts stored.","ref":"Datadog.Sketch.Store.Dense.html#total_count/1","title":"Datadog.Sketch.Store.Dense.total_count/1","type":"function"},{"doc":"iex> store = Dense.new()\n      ...> Dense.total_count(store)\n      0.0\n\n      iex> store = Dense.add(Dense.new(), 754)\n      ...> Dense.total_count(store)\n      1.0\n\n      iex> store = Dense.add_with_count(Dense.new(), 754, 42.42)\n      ...> Dense.total_count(store)\n      42.42\n\n      # Verified against golang implementation\n      iex> store = Dense.add_bins(Dense.new(), [\n      ...>   %{index: 4, count: 12.48},\n      ...>   %{index: 65, count: 12.48},\n      ...>   %{index: 37, count: 847.4}\n      ...> ])\n      ...> Dense.total_count(store)\n      872.36","ref":"Datadog.Sketch.Store.Dense.html#total_count/1-examples","title":"Examples - Datadog.Sketch.Store.Dense.total_count/1","type":"function"},{"doc":"","ref":"Datadog.Sketch.Store.Dense.html#t:t/0","title":"Datadog.Sketch.Store.Dense.t/0","type":"type"},{"doc":"# Data Streams Ex\n\nThis is a port of the [data-streams-go](https://github.com/DataDog/data-streams-go) library to Elixir.","ref":"readme.html","title":"Data Streams Ex","type":"extras"},{"doc":"This product is meant to measure end to end latency in async pipelines. It's in an Alpha phase. We currently support instrumentations of async pipelines using Kafka. Integrations with other systems will follow soon.","ref":"readme.html#introduction","title":"Introduction - Data Streams Ex","type":"extras"},{"doc":"- Data stream: A set of services connected together via *queues*\n- Pathway: A single branch of connected services\n- Queue: A connection between two services\n- Edge latency: Latency of a queue between two services\n- Latency from origin: Latency from the first tracked service, down to the current service\n- Checkpoint: records at what time a specific operation on a payload occurred (eg: The payload was sent to Kafka). The product can then measure latency between checkpoints.\n\nThe product can measure edge latency, and latency from origin, for a set of checkpoints connected together via queues.\nTo do so, we propagate timestamps, and a hash of the path that messages took with the payload.","ref":"readme.html#glossary","title":"Glossary - Data Streams Ex","type":"extras"},{"doc":"Just add [`data_streams`](https://hexdocs.pm/data_streams) to your `mix.exs` file like so:\n\n \n```elixir\ndef deps do\n  [\n    {:data_streams, \"~> 1.2.2\"}\n  ]\nend\n```\n \n\nDocumentation is automatically generated and published to [HexDocs](https://hexdocs.pm/data_streams).","ref":"readme.html#installation","title":"Installation - Data Streams Ex","type":"extras"},{"doc":"**Prerequisites**\n- Datadog Agent 7.34.0+\n- latest version of [the data streams library](https://github.com/stordco/data-streams-ex)\n\nYou will need to configure the pipeline with the trace agent URL and enable it to start on application start. This can be done via your `config/` files:\n\n```elixir\nconfig :data_streams, :pipeline,\n  enabled?: true,\n  host: \"localhost\",\n  port: 8126\n```\n\nThe host and port should point to your Datadog agent.\n\nThe instrumentation relies on creating checkpoints at various points in your data stream services with specific tags, recording the pathway that messages take along the way. For a complete picture of your services, you will also need to configure some metadata about the current service running. You can do this in your `config/` files as well:\n\n```elixir\nconfig :data_streams, :metadata,\n  service: \"my-service\",\n  env: \"production\",\n  primary_tag: \"datacenter:d1\" # You can leave this blank if you don't have a primary tag.\n```\n\nWe recommend you keep these tags matching all your other instrumentation, like Open Telemetry and `:telemetry`, to ensure Datadog can aggregate data accurately.\n\n#","ref":"readme.html#elixir-instrumentation","title":"Elixir instrumentation - Data Streams Ex","type":"extras"},{"doc":"This library contains integration modules to help integrate with various async data pipelines. See one of these modules for usage details.\n\n- `Datadog.DataStreams.Integrations.Kafka`","ref":"readme.html#integrations","title":"Integrations - Data Streams Ex","type":"extras"},{"doc":"# Changelog\n\n## [1.2.2](https://github.com/stordco/data-streams-ex/compare/v1.2.1...v1.2.2) (2023-08-10)\n\n\n#","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Sync files with stordco/common-config-elixir ([#25](https://github.com/stordco/data-streams-ex/issues/25)) ([85a61bc](https://github.com/stordco/data-streams-ex/commit/85a61bc3e0f3ce466da4b7cf2d4beb74ff334462))\n* Update dependencies ([#27](https://github.com/stordco/data-streams-ex/issues/27)) ([d16116b](https://github.com/stordco/data-streams-ex/commit/d16116bfb85898d169d9bb22ce8f427f991c85a0))\n\n## [1.2.1](https://github.com/stordco/data-streams-ex/compare/v1.2.0...v1.2.1) (2023-08-01)\n\n\n#","ref":"changelog.html#miscellaneous","title":"Miscellaneous - Changelog","type":"extras"},{"doc":"* Add elixir 1.15 to CI tests ([#21](https://github.com/stordco/data-streams-ex/issues/21)) ([d43b381](https://github.com/stordco/data-streams-ex/commit/d43b3817e7154bfb1e7f23f913343afc2d44f76c))\n* Migrate to internal FNV module ([#22](https://github.com/stordco/data-streams-ex/issues/22)) ([e3f6f91](https://github.com/stordco/data-streams-ex/commit/e3f6f9117b1d64d7db4288b79bba36fbbecaa80d))\n* Setup common config elixir ([180752d](https://github.com/stordco/data-streams-ex/commit/180752d5056e8bfff254809b78a33f3c24ef5d0e))\n* Sync files with stordco/common-config-elixir ([#23](https://github.com/stordco/data-streams-ex/issues/23)) ([a1360fd](https://github.com/stordco/data-streams-ex/commit/a1360fdcd28fbb1f908210b386f706f4adfde438))\n\n## [1.2.0](https://github.com/stordco/data-streams-ex/compare/v1.1.2...v1.2.0) (2023-07-10)\n\n\n#","ref":"changelog.html#miscellaneous","title":"Miscellaneous - Changelog","type":"extras"},{"doc":"* allow tracking kafka produce and consume offsets ([#18](https://github.com/stordco/data-streams-ex/issues/18)) ([ebab69b](https://github.com/stordco/data-streams-ex/commit/ebab69bb896eddb23913a49098ebc3460e4ae72c))\n\n## [1.1.2](https://github.com/stordco/data-streams-ex/compare/v1.1.1...v1.1.2) (2023-06-05)\n\n\n#","ref":"changelog.html#features","title":"Features - Changelog","type":"extras"},{"doc":"* defensive code around container logic ([#16](https://github.com/stordco/data-streams-ex/issues/16)) ([97d770b](https://github.com/stordco/data-streams-ex/commit/97d770b0db652f6a202bdafb812321ea55799592))\n\n## [1.1.1](https://github.com/stordco/data-streams-ex/compare/v1.1.0...v1.1.1) (2023-06-05)\n\n\n#","ref":"changelog.html#bug-fixes","title":"Bug Fixes - Changelog","type":"extras"},{"doc":"* set span attribute to pathway hash ([#14](https://github.com/stordco/data-streams-ex/issues/14)) ([94ca3fd](https://github.com/stordco/data-streams-ex/commit/94ca3fd02f7afa4434664463bc8b257020f48243))\n\n## [1.1.0](https://github.com/stordco/data-streams-ex/compare/v1.0.0...v1.1.0) (2023-05-23)\n\n\n#","ref":"changelog.html#bug-fixes","title":"Bug Fixes - Changelog","type":"extras"},{"doc":"* add container id to transport headers ([#12](https://github.com/stordco/data-streams-ex/issues/12)) ([4d58826](https://github.com/stordco/data-streams-ex/commit/4d588260c559babbbd8635ca9b8d8261d4541d61))","ref":"changelog.html#features","title":"Features - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-2023-04-06","title":"1.0.0 (2023-04-06) - Changelog","type":"extras"},{"doc":"* Package and application configuration is now under `data_streams` instead of `dd_data_streams`\n\n#","ref":"changelog.html#breaking-changes","title":"⚠ BREAKING CHANGES - Changelog","type":"extras"},{"doc":"* add basic implementation of ddsketch ([#1](https://github.com/stordco/data-streams-ex/issues/1)) ([125b5ed](https://github.com/stordco/data-streams-ex/commit/125b5ed57fb3b407406339d050121ff052aabf4a))\n* add basic kafka tracking support with data streams ([#3](https://github.com/stordco/data-streams-ex/issues/3)) ([bfc6a0b](https://github.com/stordco/data-streams-ex/commit/bfc6a0b88879c8469760a521e5e0408d617e6918))\n* add LICENSE file ([#11](https://github.com/stordco/data-streams-ex/issues/11)) ([6c5668f](https://github.com/stordco/data-streams-ex/commit/6c5668f612e89204a05e03f998a655324b2b6d8d))\n* link open telemetry span to current pathway context ([#5](https://github.com/stordco/data-streams-ex/issues/5)) ([e0ed9b2](https://github.com/stordco/data-streams-ex/commit/e0ed9b22b920c12e11684fc5eef9e619eb05a1aa))\n* rename dd_data_streams to data_streams ([#9](https://github.com/stordco/data-streams-ex/issues/9)) ([a0d1742](https://github.com/stordco/data-streams-ex/commit/a0d1742f9a51e45608fde92918a4c1b6c777cf9d))\n\n\n#","ref":"changelog.html#features","title":"Features - Changelog","type":"extras"},{"doc":"* add case for error http status ([#8](https://github.com/stordco/data-streams-ex/issues/8)) ([ef4a95d](https://github.com/stordco/data-streams-ex/commit/ef4a95dea09a644c9d90079a96d2f2683b8c6aa5))\n* **ci:** update PR title regex check ([64ef99f](https://github.com/stordco/data-streams-ex/commit/64ef99fa5e8c4674debb43242307019600cc3060))\n* dialyzer warnings for kafka integration map ([18bf936](https://github.com/stordco/data-streams-ex/commit/18bf936c1249360042a4d3e1434bf0a94236ed0b))\n* filter out nil values from kafka integration tags ([b33926f](https://github.com/stordco/data-streams-ex/commit/b33926f584869165c0340fe06bf088813c80fda7))\n* update kafka integration to not set context on produce ([#7](https://github.com/stordco/data-streams-ex/issues/7)) ([6807b6d](https://github.com/stordco/data-streams-ex/commit/6807b6dcd1f0504179e24ad78062c4484b4fe2c3))\n* update otel resource service configuration ([adb9890](https://github.com/stordco/data-streams-ex/commit/adb9890f19dd49b2d3b1a4da6e00bf6d5f6823da))\n* update tag logic to be more consistant ([#4](https://github.com/stordco/data-streams-ex/issues/4)) ([48d13df](https://github.com/stordco/data-streams-ex/commit/48d13dfa5634f24a83eba2e3b0a9a09adbced4a4))","ref":"changelog.html#bug-fixes","title":"Bug Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#changelog","title":"Changelog - Changelog","type":"extras"}]}